@@ -1,7 +1,7 @@
 #
 # api.py
 #
-# Copyright (C) 2017-2020 frnmst (Franco Masotti) <franco.masotti@live.com>
+# Copyright (C) 2017-2021 frnmst (Franco Masotti) <franco.masotti@live.com>
 #
 # This file is part of md-toc.
 #
@@ -134,7 +134,8 @@ def build_toc(filename: str,
               keep_header_levels: int = 3,
               parser: str = 'github',
               list_marker: str = '-',
-              skip_lines: int = 0) -> str:
+              skip_lines: int = 0,
+              constant_ordered_list: bool = False) -> str:
     r"""Build the table of contents of a single file.
 
     :parameter filename: the file that needs to be read.
@@ -149,9 +150,14 @@ def build_toc(filename: str,
          Defaults to ``3``.
     :parameter parser: decides rules on how to generate anchor links.
          Defaults to ``github``.
+    :parameter list_marker: a string that contains some of the first
+         characters of the list element.
+         Defaults to ``-``.
     :parameter skip_lines: the number of lines to be skipped from
          the start of file before parsing for table of contents.
          Defaults to ``0```.
+    :parameter constant_ordered_list: use a single integer
+        as list marker. This sets ordered to ``True``.
     :type filename: str
     :type ordered: bool
     :type no_links: bool
@@ -160,6 +166,7 @@ def build_toc(filename: str,
     :type parser: str
     :type list_marker: str
     :type skip_lines: int
+    :type constant_ordered_list: bool
     :returns: toc, the corresponding table of contents of the file.
     :rtype: str
     :raises: a built-in exception.
@@ -187,6 +194,10 @@ def build_toc(filename: str,
     if (ordered and list_marker == md_parser[parser]['list']['unordered']['default marker']):
         list_marker = md_parser[parser]['list']['ordered'][
             'default closing marker']
+    if constant_ordered_list:
+        ordered = True
+    if ordered and (list_marker is None or list_marker not in md_parser[parser]['list']['ordered']['closing markers']):
+        list_marker = md_parser[parser]['list']['ordered']['default closing marker']
 
     if skip_lines > 0:
         loop = True
@@ -239,11 +250,14 @@ def build_toc(filename: str,
                 header_type_curr = header['type']
 
                 # Take care of the ordered TOC.
-                if ordered:
+                if ordered and not constant_ordered_list:
                     increase_index_ordered_list(header_type_counter,
                                                 header_type_prev,
                                                 header_type_curr, parser)
                     index = header_type_counter[header_type_curr]
+                elif constant_ordered_list:
+                    # This value should work on most parsers.
+                    index = 1
                 else:
                     index = 1
 
@@ -298,7 +312,8 @@ def build_multiple_tocs(filenames: list,
                         keep_header_levels: int = 3,
                         parser: str = 'github',
                         list_marker: str = '-',
-                        skip_lines: int = 0) -> list:
+                        skip_lines: int = 0,
+                        constant_ordered_list: bool = False) -> list:
     r"""Parse files by line and build the table of contents of each file.
 
     :parameter filenames: the files that needs to be read.
@@ -313,12 +328,23 @@ def build_multiple_tocs(filenames: list,
          Defaults to ``3``.
     :parameter parser: decides rules on how to generate anchor links.
          Defaults to ``github``.
+    :parameter skip_lines: the number of lines to be skipped from
+         the start of file before parsing for table of contents.
+         Defaults to ``0```.
+    :parameter list_marker: a string that contains some of the first
+         characters of the list element.
+         Defaults to ``-``.
+    :parameter constant_ordered_list: use a single integer
+        as list marker. This sets ordered to ``True``.
     :type filenames: list
     :type ordered: bool
     :type no_links: bool
     :type no_indentation: bool
     :type keep_header_levels: int
     :type parser: str
+    :type list_marker: str
+    :type skip_lines: int
+    :type constant_ordered_list: bool
     :returns: toc_struct, the corresponding table of contents for each input
          file.
     :rtype: list
@@ -340,7 +366,7 @@ def build_multiple_tocs(filenames: list,
         toc_struct.append(
             build_toc(filenames[file_id], ordered, no_links, no_indentation,
                       no_list_coherence, keep_header_levels, parser,
-                      list_marker, skip_lines))
+                      list_marker, skip_lines, constant_ordered_list))
         file_id += 1
 
     return toc_struct
@@ -807,27 +833,40 @@ def get_atx_heading(line: str,
         # are not at the end of the line.
         # See the two CRLF marker tests.
         crlf_marker = 0
-        while i < len_line - cs_start - 1:
+        stripped_crlf = False
+        while i < len_line - cs_start:
             if line_prime[i] in ['\u000a', '\u000d']:
                 crlf_marker = i
+                stripped_crlf = True
             i += 1
 
+        # crlf_marker is the first CR LF character in the string.
         i = crlf_marker
-        while i < len_line - cs_start - 1 and go:
-            if (line_prime[i] not in [' ', '#', '\u000a', '\u000d']
+        if stripped_crlf:
+            # Skip last character only if is '\u000a', '\u000d'.
+            i += 1
+
+        # We know for sure that from now '\u000a', '\u000d' will not be
+        # considered.
+        cs_end = i
+
+        # Cut spaces and hashes.
+        while go and i < len_line - cs_start:
+            if (line_prime[i] not in [' ', '#']
                     or hash_char_rounds > 1):
-                if i > hash_round_start:
+                if i > hash_round_start and hash_char_rounds > 0:
                     cs_end = len_line - hash_round_start
                 else:
                     cs_end = len_line - i
                 go = False
-            while go and line_prime[i] in [' ', '\u000a', '\u000d']:
-                i += 1
-            hash_round_start = i
-            while go and line_prime[i] == '#':
-                i += 1
-            if i > hash_round_start:
-                hash_char_rounds += 1
+            if go:
+                while line_prime[i] == ' ':
+                    i += 1
+                hash_round_start = i
+                while line_prime[i] == '#':
+                    i += 1
+                if i > hash_round_start:
+                    hash_char_rounds += 1
 
         final_line = line[cs_start:cs_end]
 
